//
/*! libresignin
*/
// Copyright 2022, Jesús Pérez Lorenzo
//
use axum::{
	response::Html,
	routing::get,
	routing::post,
	Router,
	// extract::{self,Extension,Path,Query},
	extract::{Extension,Path},
	http::{header::HeaderMap, StatusCode},
  // body::{Bytes, Body},
	response::IntoResponse,
	BoxError,
};
use std::{
  borrow::Cow,
};
use crate::defs::{DataDBs}; 
use crate::utils::reqenv::ReqEnv;

use app_env::{
  // AppStore,
  // appenv::AppEnv,
  // appinfo::AppInfo,
  // appdata::AppData,
  //config::{WebServer,SigninServer,SigninAuthzRoutes,AuthzMode},
  config::{SigninAuthzRoute,AuthzMode},
};

// pub async fn html_handler(Path(path): Path<String>,query: Option<Query<String>>, Extension(dbs): Extension<DataDBs>) -> Html<&'static str> {
// pub async fn html_handler(Path(path): Path<String>,Extension(dbs): Extension<DataDBs>) -> Html<&'static str> {
pub async fn html_handler(header: HeaderMap, Path(name): Path<String>,Extension(dbs): Extension<DataDBs>) -> Html<&'static str> {
 let reqenv = ReqEnv::new(dbs.app, dbs.auth, header, "get", "/hello", "html", "rt");
  dbg!("{:#?}",reqenv.user_authentication().await);
// dbg!("{:#?}",headers);
//dbg!("{:#?}",name);	
//dbg!("{:#?}",reqenv.config());	
// 	dbg!("{:#?}",query);	
// 	dbg!("{:#?}",dbs.auth.users);	
  Html("<h1>Hello, World!</h1>")
}
pub async fn alive_handler() -> Html<&'static str> {
    Html("ok")
}
pub async fn ready_handler() -> Html<&'static str> {
    Html("ok")
}
pub async fn handle_error(error: BoxError) -> impl IntoResponse {
    if error.is::<tower::timeout::error::Elapsed>() {
        return (StatusCode::REQUEST_TIMEOUT, Cow::from("request timed out"));
    }
    if error.is::<tower::load_shed::error::Overloaded>() {
        return (
            StatusCode::SERVICE_UNAVAILABLE,
            Cow::from("service is overloaded, try again later"),
        );
    }
    (
        StatusCode::INTERNAL_SERVER_ERROR,
        Cow::from(format!("Unhandled internal error: {}", error)),
    )
}
pub fn router_handlers(web_router: Router) -> Router {
	let mut webrouter = web_router.to_owned();
	webrouter = crate::handlers::kratos::def_handlers(webrouter);
  webrouter
  .route("/getjwt", get(crate::handlers::jwt::getjwt))
  .route("/protected", get(crate::handlers::jwt::protected))
  .route("/authorize", post(crate::handlers::jwt::authorize))
  .route("/session", get(crate::handlers::sessions::handler))
  .route("/hello/:name", get(html_handler))
	.route("/health/alive", get(alive_handler))
  .route("/health/ready", get(ready_handler))
}
// These are to implement configuration defined signin authz routes 
// using authz-route "path" ,"mode"(AuthMode) and "handler"
// 
fn add_kratos_handler(web_router: Router,route: SigninAuthzRoute) -> Router {
	let mut webrouter = web_router.to_owned();
	if route.path.is_empty() {
    return webrouter;
	}
	match route.handler.as_str() {
		// TODO Add handlers for Kratos. This should be moved from String to Enum type in SinginAuthRoute
	 "headers" => webrouter = webrouter.route(route.path.as_str(), get(crate::handlers::extauthz::authz_handler)),
	  _ => { 
			println!("Kratos signin authz route ({}) path ({}) with handler ({}) use authz_handler",route.id,route.path,route.handler);
	    webrouter = webrouter.route(route.path.as_str(), get(crate::handlers::extauthz::authz_handler));
		},
	};
	webrouter
}
pub fn router_authz_handlers(web_router: Router,signin_auth_routes: Vec<SigninAuthzRoute>) -> Router {
	let mut webrouter = web_router.to_owned();
	for route in signin_auth_routes {
		match route.mode {
			AuthzMode::Kratos => webrouter = add_kratos_handler(webrouter,route),
		 // TODO Add new AuthMode
			_ => eprintln!("Signin authz route {} undefined mode {}",route.id,route.mode),
		}
	}
	webrouter
}